#include "pnnpsolvert.h"
#include "pglobalt.h"
#include "seqfilest.h"
#include <string.h>
#include <math.h>
#include "mpi.h"

void par_solve_nonlinear_nonstationary_problem ()
{
  long i,j,k,n,ini;
  double dt,end_time,alpha,norfb,err,totnorfb;
  double *d,*fb,*fi,*p,*lhs,*lhsi,*tdlhs,*rhs,*arhs;
  MPI_Status stat;

  n=Ndoft;
    
  lhs = Lsrst->give_lhs (0);
  tdlhs = Lsrst->give_tdlhs (0);
  lhsi = Lsrst->lhsi;
  rhs = Lsrst->give_rhs (0);
  
  d = new double [n];
  arhs = new double [n];
  p = new double [n];
  fb = new double [n];
  fi = new double [n];
  
  //  initial values
  nullv (lhs,n);
  nullv (tdlhs,n);
  nullv (rhs,n);
  nullv (arhs,n);
  nullv (d,n);
  nullv (p,n);
  nullv (fb,n);
  nullv (fi,n);
  
  //  nodes - integration points interpolation
  approximation ();
  
  alpha=Tp->alpha;
  err=Tp->err;
  ini=Tp->nii;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  print_initt(-1, "wt",Ptp->fni,Ptp->fei);
  print_stept(0,i,Tp->time,NULL);
  print_flusht();
  
  long nidof=Psolt->schcom[0].nidof;
  long nrdof=Psolt->schcom[0].nbdof;
  double *uf,*buff;
  buff = new double [10];
  uf = new double [10];
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  i=0;
  do{
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n iteration number %ld, time %f, time step = %f\n",i,Tp->time,dt);
          
    //kontrolni tisk ???!!!
    /* if ((Tp->time >= 304.0))
       {
       fprintf (Outt,"\n\nkontrolni tisk time = %e\n",Tp->time);
       fprintf (Outt,"------------------------\n\n",Tp->time);	
       }
    */
    //  capacity matrix
    capacity_matrix (0);
    
    //  conductivity matrix
    conductivity_matrix (0);
    
    for (j=0;j<n;j++){
      d[j] = lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  right hand side
    trfel_right_hand_side (0,rhs,n);
    
    //fprintf (Outt,"\n\n\n\n\n kontrola prave strany");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Outt,"\n %ld %le",ijk,rhs[ijk]);
    //}
    
    
    
    //  auxiliary vector  K (d+(1-alpha)*dt*v)
    Kmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    //  F - K (d+(1-alpha)*dt*v)
    for (j=0;j<n;j++){
      fb[j] = rhs[j] - p[j];
    }
    
    //Kmat->solve_system (tdlhs,fb);
    Psolt->par_linear_solver (Gtt,Kmat,tdlhs,fb,Outt,Mesprt);
    
    //  d_{i+1}=\tilda{d_{i+1}} + alpha.dt.v_{i+1}
    for (j=0;j<n;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }
    
    //fprintf (Outt,"\n\n\n\n\n kontrola leve strany");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Outt,"\n %ld %le",ijk,lhs[ijk]);
    //}

    approximation ();
    internal_fluxes (fi,n);
    
    //  vector of unbalanced fluxes
    for (j=0;j<n;j++){
      fb[j]=fi[j];
      //fprintf (Outt,"\nfb %ld  %e",j,fb[j]);
    }

    
    /*
    //norfb = ss (fb,fb,n);
    norfb = ss (fb,fb,nidof);
    long kk=0;
    for (k=nidof;k<n;k++){
      buff[kk]=fb[k];
      kk++;
    }
    
    
    if (Myrank==0){
      totnorfb=norfb;
      for (j=1;j<Nproc;j++){
	MPI_Recv (&norfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	totnorfb+=norfb;
      }
    }
    else{
      MPI_Send(&norfb,1,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
    }
    MPI_Barrier (MPI_COMM_WORLD);

    if (Myrank==0){
      for (k=0;k<6;k++){
	uf[k]=buff[k];
	fprintf (Outt,"\nbuf %ld  %e",k,buff[k]);
      }
      for (j=1;j<Nproc;j++){
	MPI_Recv (buff,10,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	for (k=0;k<6;k++){
	fprintf (Outt,"\nbuf %ld  %e",k,buff[k]);
	  uf[k]+=buff[k];
	}
      }
      for (j=0;j<6;j++){
	fprintf (Outt,"\nuf %ld  %e",j,uf[j]);
      }
      totnorfb += ss (uf,uf,6);
      
      for (j=1;j<Nproc;j++){
	MPI_Send(&totnorfb,1,MPI_DOUBLE,j,Myrank,MPI_COMM_WORLD);
      }
    }
    else{
      MPI_Send(buff,10,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
      MPI_Recv (&totnorfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
    }
    MPI_Barrier (MPI_COMM_WORLD);
    */

    /*
    if (Myrank==0){
      totnorfb=norfb;
      for (j=1;j<Nproc;j++){
	MPI_Recv (&norfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	totnorfb+=norfb;
      }
      for (j=1;j<Nproc;j++){
	MPI_Send(&totnorfb,1,MPI_DOUBLE,j,Myrank,MPI_COMM_WORLD);
      }
    }
    else{
      MPI_Send(&norfb,1,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
      MPI_Recv (&totnorfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
    }
    MPI_Barrier (MPI_COMM_WORLD);
    */
    
    
    
    
    totnorfb = Psolt->unbalanced_values (fb,Outt);
    
    
    

    if (Mesprt==1)  fprintf (stdout,"\n%e %e",totnorfb,err);
    //if (Mesprt==1)  fprintf (Outt,"\n\ntotnorfb %e   err %e\n",totnorfb,err);
    
    if (totnorfb<err){
      //  time increment
      Tp->time = Tp->timecont.newtime ();
      dt = Tp->timecont.actualbacktimeincr ();
      i++;

      print_stept(0,i,Tp->time,NULL);
      print_flusht();
      continue;
    }
    

    for (j=0;j<ini;j++){
      

      //fprintf (Outt,"\n\n\n\n\n vnitrni smycka kontrola prave strany");
      //for (long ijk=0;ijk<n;ijk++){
      //fprintf (Outt,"\n %ld %le",ijk,fb[ijk]);
      //}

      //Kmat->solve_system (p,fb);
      Psolt->par_linear_solver (Gtt,Kmat,p,fb,Outt,Mesprt);
      
      for (k=0;k<n;k++){
	tdlhs[k]+=p[k];
	lhs[k]+=alpha*dt*p[k];
      }
      
      //fprintf (Outt,"\n\n\n\n\n vnitrni smycka kontrola leve strany");
      //for (long ijk=0;ijk<n;ijk++){
      //fprintf (Outt,"\n %ld %le",ijk,lhs[ijk]);
      //}

      approximation ();
      internal_fluxes (fi,n);
      
      //  vector of unbalanced fluxes
      for (k=0;k<n;k++){
	fb[k]= fi[k];
      }
      
      //norfb = ss (fb,fb,n);
      /*
      norfb = ss (fb,fb,nidof);
      long kk=0;
      for (k=nidof;k<n;k++){
	buff[kk]=fb[k];
	kk++;
      }
      
      
      if (Myrank==0){
	totnorfb=norfb;
	for (k=1;k<Nproc;k++){
	  MPI_Recv (&norfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	  totnorfb+=norfb;
	}
      }
      else{
	MPI_Send(&norfb,1,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
      }
      MPI_Barrier (MPI_COMM_WORLD);
      
      if (Myrank==0){
	for (k=0;k<10;k++){
	  uf[k]=buff[k];
	}
	for (k=1;k<Nproc;k++){
	  MPI_Recv (buff,10,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	  for (kk=0;kk<10;kk++){
	    uf[kk]+=buff[kk];
	  }
	}
	totnorfb += ss (uf,uf,6);
	
	for (k=1;k<Nproc;k++){
	  MPI_Send(&totnorfb,1,MPI_DOUBLE,k,Myrank,MPI_COMM_WORLD);
	}
      }
      else{
	MPI_Send(buff,10,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
	MPI_Recv (&totnorfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
      }
      MPI_Barrier (MPI_COMM_WORLD);
      

      if (Myrank==0){
	totnorfb=norfb;
	for (k=1;k<Nproc;k++){
	  MPI_Recv (&norfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
	  totnorfb+=norfb;
	}
	for (k=1;k<Nproc;k++){
	  MPI_Send(&totnorfb,1,MPI_DOUBLE,k,Myrank,MPI_COMM_WORLD);
	}
      }
      else{
	MPI_Send(&norfb,1,MPI_DOUBLE,0,Myrank,MPI_COMM_WORLD);
	MPI_Recv (&totnorfb,1,MPI_DOUBLE,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,&stat);
      }
      MPI_Barrier (MPI_COMM_WORLD);
      */
      
      
       
      
      totnorfb = Psolt->unbalanced_values (fb,Outt);
      
      if (Mesprt==1)  fprintf (stdout,"\n iteration %ld   error %e",j,totnorfb);
      
      if (totnorfb<err){
	break;
      }
    }


    //  time increment
    Tp->time = Tp->timecont.newtime ();
    dt = Tp->timecont.actualbacktimeincr ();
    i++;
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();

    printf ("\n Tp->time   %e",Tp->time);
    printf ("\n i          %ld",i);
    
  }while(Tp->time<end_time);
  
  delete [] fi;  delete [] fb;  delete [] p;
  delete [] arhs;  delete [] d;
  
  //print_closet();
}
