#include "ptrfelinit.h"
#include "pglobalt.h"
#include "../../GEFEL/xfile.h"

void ptrfel_init (int argc,char *argv)
{
  char name[1100];
  XFILE *in;
  
  //  creation of main objects
  Tp = new probdesct;
  Ptp = new pprobdesct;
  Gtt = new gtopology;
  Tt = new transtop;
  Tm = new transmat;
  Tc = new transcrsec;
  Tb = new transbclc;
  Outdt = new outdrivert;

  
  //  names of input files
  strcpy (name,argv);
  sprintf (&name[strlen (argv)],"%d.in",Myrank+1);
  in = xfopen (name,"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file is not open on processor %d.\n",Myrank);
  }
  in->maxlnsize = 1000;
  in->warrning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;

  //  number of subdomain
  xfscanf (in,"%ld",&Ndom);
  Ndom--;

  //  basic problem data
  Tp->read (in);

  //  creation of additional object
  Psolt = new psolver (Nproc,Myrank);

  //  reading data about parallel solver
  Psolt->read (in,Ndom,Tp->tstorkm,Mesprt);
  

  char buf[1001];
  strcpy (buf,argv);
  sprintf (&buf[strlen (argv)],"%d.log",Myrank+1);
  Outt = fopen (buf,"w");
  if (Outt==NULL){
    fprintf (stderr,"\n\n output file is not open on processor %d.\n",Myrank);
  }
 
  Psolt->procdomcorr ();

  
  //  input of node and element data
  Tt->read (in);

  //  initiation of parallel solver
  Psolt->initiate (Gtt);

  //  reading of the array ltg (local-to-global correspondence)
  Psolt->read_ltg (in,Gtt,Outt);

  fprintf (Outt,"\n Nproc  %d",Nproc);
  fprintf (Outt,"\n Ndom   %d",Ndom);
  fprintf (Outt,"\n Myrank %d",Myrank);

  //*************************
  //  code numbers generation
  //*************************
  Ndoft=Psolt->ordering (Gtt,Outt);

  //  input of material characteristics
  Tm->read (in);
  
  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();

  //  input of cross section characteristics
  Tc->read (in);
  
  //  input of boundary conditions
  Tb->read (in);
  
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();
  
  
  //  global vectors allocations
  Lsrst = new lhsrhst;
  Lsrst->alloc ();
  
  Lsrst->initcond (in);
  
  // input of output description
  Outdt->read(in);

  
  //  names of output files
  strcpy (name,Outdt->outfn);
  sprintf (&name[strlen (Outdt->outfn)],"%d.out",Myrank+1);
  strcpy (Outdt->outfn, name);
  if (Outdt->gf != grftt_no)
  {
    strcpy (name,Outdt->outgrfn);
    sprintf (&name[strlen (Outdt->outgrfn)],"%d",Myrank+1);
    strcpy (Outdt->outgrfn, name);
  }
  if (Outdt->ndiag > 0)
  {
    strcpy (name,Outdt->outdiagfn);
    sprintf (&name[strlen (Outdt->outdiagfn)],"%d.dat",Myrank+1);
    strcpy (Outdt->outdiagfn, name);
  }  

  fflush(stdout);
  fflush(stderr);

  Ptp->shift_indices ();
  xfclose (in);
}
